from air import Jinja2Renderer

from fastapi import FastAPI, Request
from fastapi.testclient import TestClient


def test_Jinja2Renderer():
    """Test the Jinja2Renderer class."""
    app = FastAPI()

    render = Jinja2Renderer(directory="tests/templates")

    @app.get("/test")
    def test_endpoint(request: Request):
        return render(
            request,
            name="home.html",
            context={"title": "Test Page", "content": "Hello, World!"},
        )

    client = TestClient(app)
    response = client.get("/test")

    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert (
        response.text
        == "<html>\n<title>Test Page</title>\n<h1>Hello, World!</h1>\n</html>"
    )


def test_Jinja2Renderer_no_context():
    """Test the Jinja2Renderer class."""
    app = FastAPI()

    render = Jinja2Renderer(directory="tests/templates")

    @app.get("/test")
    def test_endpoint(request: Request):
        return render(request, name="home.html")

    client = TestClient(app)
    response = client.get("/test")

    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert response.text == "<html>\n<title></title>\n<h1></h1>\n</html>"



import typer
from air.generator import StaticSiteGenerator
from air.plugins.markdown_plugin import MarkdownPlugin
from pathlib import Path
from typing_extensions import Annotated

app = typer.Typer()


@app.command()
def html(
    source_dir: Annotated[Path, typer.Argument()] = Path("input"),
    output_dir: Annotated[Path, typer.Argument()] = Path("public"),
) -> int:
    """Build a static site from source files."""
    print("Building site...")
    generator = StaticSiteGenerator(source_dir, output_dir)
    generator.register_plugin(MarkdownPlugin)
    generator.build()
    print(f"Site built from {source_dir}/ to {output_dir}/")
    return 0



import air


def test_atag_no_attrs_no_children():
    assert air.A().render() == "<a></a>"


def test_atag_yes_attrs_no_children():
    tag = air.A(href="/", cls="link").render()
    assert tag == '<a href="/" class="link"></a>'


def test_atag_yes_attrs_text_children():
    tag = air.A("Link here", href="/", cls="link").render()
    assert tag == '<a href="/" class="link">Link here</a>'


def test_divtag_yes_attrs_a_child():
    html = air.Div(air.A("Link here", href="/", cls="link")).render()
    assert html == '<div><a href="/" class="link">Link here</a></div>'


def test_divtag_yes_attrs_multiple_a_children():
    html = air.Div(
        air.A("Link here", href="/", cls="link"),
        air.A("Another link", href="/", cls="timid"),
    ).render()
    assert (
        html
        == '<div><a href="/" class="link">Link here</a><a href="/" class="timid">Another link</a></div>'
    )


def test_divtag_yes_attrs_nested_children():
    html = air.Div(
        air.P(
            "Links are here",
            air.A("Link here", href="/", cls="link"),
            air.A("Another link", href="/", cls="timid"),
        )
    ).render()
    assert (
        html
        == '<div><p>Links are here<a href="/" class="link">Link here</a><a href="/" class="timid">Another link</a></p></div>'
    )


def test_name_types():
    assert issubclass(air.A, air.Tag)
    assert issubclass(air.Div, air.Tag)
    assert issubclass(air.P, air.Tag)


def test_subclassing():
    class AwesomeP(air.P):
        def render(self) -> str:
            return f"<p{self.attrs}>AWESOME {self.children}!</p>"

    assert AwesomeP("library").render() == "<p>AWESOME library!</p>"


def test_subclassing_nested():
    class AwesomeP(air.P):
        def render(self) -> str:
            return f"<p{self.attrs}>AWESOME {self.children}!</p>"

    html = air.Div(AwesomeP("library")).render()
    assert html == "<div><p>AWESOME library!</p></div>"


def test_text_child_with_sibling_elements():
    html = air.P("This is a", air.Strong("cut off"), "sentence").render()
    assert html == "<p>This is a<strong>cut off</strong>sentence</p>"


def test_special_attributes():
    html = air.P("Has a special attribute", **{"@fun": "times ahead"}).render()
    assert html == '<p @fun="times ahead">Has a special attribute</p>'

    html = air.P("Has a special attribute", **{"!data": "12345"}).render()
    assert html == '<p !data="12345">Has a special attribute</p>'

    html = air.P("HTMX example", hx_post="/get", _id="53").render()
    assert html == '<p hx-post="/get" id="53">HTMX example</p>'


def test_htmx_render():
    html = air.Htmx(air.P("Hello, world")).render()
    assert (
        html
        == '<!doctype html><html><head><script src="https://cdn.jsdelivr.net/npm/htmx.org@2.0.5/dist/htmx.min.js"></script></head><body><p>Hello, world</p></body></html>'
    )


def test_raw_html_basic():
    """Test basic RawHTML rendering without escaping."""
    raw = air.RawHTML("<strong>Bold</strong> & <em>italic</em>")
    assert raw.render() == "<strong>Bold</strong> & <em>italic</em>"


def test_raw_html_with_script():
    """Test that RawHTML does not escape script tags (security risk)."""
    raw = air.RawHTML('<script>alert("XSS")</script>')
    assert raw.render() == '<script>alert("XSS")</script>'
    # This test documents the security risk


def test_raw_html_invalid_args():
    """Test that RawHTML raises errors with invalid arguments."""
    try:
        air.RawHTML("first", "second")
        assert False, "Expected ValueError"
    except ValueError as e:
        assert "RawHTML accepts only one string argument" in str(e)

    try:
        air.RawHTML(123)
        assert False, "Expected TypeError"
    except TypeError as e:
        assert "RawHTML only accepts string content" in str(e)

    try:
        air.RawHTML(air.Div("test"))
        assert False, "Expected TypeError"
    except TypeError as e:
        assert "RawHTML only accepts string content" in str(e)


def test_raw_html_ignores_kwargs():
    """Test that RawHTML ignores keyword arguments."""
    raw = air.RawHTML("<div>Test</div>", id="ignored", cls="also-ignored")
    assert raw.render() == "<div>Test</div>"



import re
import shutil
from pathlib import Path
from jinja2 import Environment, FileSystemLoader, select_autoescape
from typing import List


class Plugin:
    def __init__(self, generator) -> None:
        self.generator = generator

    def run(self) -> None:
        pass


class StaticSiteGenerator:
    def __init__(self, source_dir: Path, output_dir: Path) -> None:
        self.source_dir = source_dir
        self.output_dir = output_dir
        self.env = Environment(
            loader=FileSystemLoader(str(self.source_dir)),
            autoescape=select_autoescape(["html"]),
        )
        self.plugins: List[Plugin] = []

    def register_plugin(self, plugin: type) -> None:
        plugin_instance = plugin(self)
        self.plugins.append(plugin_instance)

    def build(self) -> None:
        if self.output_dir.exists():
            shutil.rmtree(self.output_dir)
        self.output_dir.mkdir(parents=True)

        base_pattern = re.compile(r"^base.*\.html$")

        for path in self.source_dir.rglob("*.html"):
            relative_path = path.relative_to(self.source_dir)
            output_path = self.output_dir / relative_path

            # Skip rendering templates that start with "base"
            if base_pattern.match(path.name):
                continue

            output_path.parent.mkdir(parents=True, exist_ok=True)

            with open(output_path, "w", encoding="utf-8") as f:
                template = self.env.get_template(str(relative_path))
                f.write(template.render())

        for plugin in self.plugins:
            plugin.run()



from .tags import CaseTag


class A(CaseTag):
    pass


class Animate(CaseTag):
    pass


class AnimateMotion(CaseTag):
    pass


class AnimateTransform(CaseTag):
    pass


class Circle(CaseTag):
    pass


class ClipPath(CaseTag):
    pass


class Defs(CaseTag):
    pass


class Desc(CaseTag):
    pass


class Ellipse(CaseTag):
    pass


class FeBlend(CaseTag):
    pass


class FeColorMatrix(CaseTag):
    pass


class FeComponentTransfer(CaseTag):
    pass


class FeComposite(CaseTag):
    pass


class FeConvolveMatrix(CaseTag):
    pass


class FeDiffuseLighting(CaseTag):
    pass


class FeDisplacementMap(CaseTag):
    pass


class FeDistantLight(CaseTag):
    pass


class FeDropShadow(CaseTag):
    pass


class FeFlood(CaseTag):
    pass


class FeFuncA(CaseTag):
    pass


class FeFuncB(CaseTag):
    pass


class FeFuncG(CaseTag):
    pass


class FeFuncR(CaseTag):
    pass


class FeGaussianBlur(CaseTag):
    pass


class FeImage(CaseTag):
    pass


class FeMerge(CaseTag):
    pass


class FeMergeNode(CaseTag):
    pass


class FeMorphology(CaseTag):
    pass


class FeOffset(CaseTag):
    pass


class FePointLight(CaseTag):
    pass


class FeSpecularLighting(CaseTag):
    pass


class FeSpotLight(CaseTag):
    pass


class FeTile(CaseTag):
    pass


class FeTurbulence(CaseTag):
    pass


class Filter(CaseTag):
    pass


class ForeignObject(CaseTag):
    pass


class G(CaseTag):
    pass


class Image(CaseTag):
    pass


class Line(CaseTag):
    pass


class LinearGradient(CaseTag):
    pass


class Marker(CaseTag):
    pass


class Mask(CaseTag):
    pass


class Metadata(CaseTag):
    pass


class Mpath(CaseTag):
    pass


class Path(CaseTag):
    pass


class Pattern(CaseTag):
    pass


class Polygon(CaseTag):
    pass


class Polyline(CaseTag):
    pass


class RadialGradient(CaseTag):
    pass


class Rect(CaseTag):
    pass


class Script(CaseTag):
    pass


class Set(CaseTag):
    pass


class Stop(CaseTag):
    pass


class Style(CaseTag):
    pass


class Svg(CaseTag):
    pass


class Switch(CaseTag):
    pass


class Symbol(CaseTag):
    pass


class Text(CaseTag):
    pass


class TextPath(CaseTag):
    pass


class Title(CaseTag):
    pass


class Tspan(CaseTag):
    pass


class Use(CaseTag):
    pass


class View(CaseTag):
    pass
